/* libdf3 - common.c
 * utilities + helpers.
 */

#include "df3_priv.h"

#include <sys/stat.h>
#include <unistd.h>


/* handles have a simple checksum to allow easy checking
 * whether they're kosher, filled in by open + create,
 * r/o for comparisons thereafter.
 * TODO include dp.
 * TODO fewer magic numbers.
 */

uint16_t df3CalcChecksum(df3Handle const * const df3) {
  static int const mult[4] = {31, 23, 79, 47};
  size_t n = 0;
  int s = 0;
  for (size_t i = 0; df3->fname[i]; ++i)
    s += df3->fname[i] * mult[n++ % 4];
  s += (int)df3->axes.x * mult[n++ % 4];
  s += (int)df3->axes.y * mult[n++ % 4];
  s += (int)df3->axes.z * mult[n++ % 4];
  s += (int)df3->bpv * mult[n++ % 4];
  s += df3->readonly * mult[n++ % 4];
  s += df3->ismapped * mult[n % 4];
  return (uint16_t)(UINT16_MAX & s);
}


bool df3ValidHandle(df3Handle const * const df3) {
  /* axes, bpv + fname look sane?  dp set? */
  if (!df3_check_axes(df3->axes)  ||  !df3_check_bpv(df3->bpv))
    return false;
  else if (!df3_check_fname(df3->fname))
    return false;
  else if (!df3->dp)
    return false;
  /* checksum tallies? */
  return df3CalcChecksum(df3) == df3->chcks;
}


/* check coord is inside volume */
bool df3ValidAt(df3Axes const d, df3Coord const at) {
  return at.x < d.x  &&  at.y < d.y  &&  at.z < d.z;
}


/* test whether file exists, return its size via pointer,
 * 0 if not exists (or empty!).
 * returns 0 if successful, or errno.
 */

int df3ExistFileCheck(char const * const fn, off_t *sz) {
  errno = 0;
  *sz = 0;
  struct stat sb;
  int rc = stat(fn, &sb);
  if (0 == rc)
    *sz = sb.st_size;
  else if (ENOENT == errno)
    rc = 0;
  else 
    rc = errno;
  return rc;
}


/* calculate offset for next read/write.
 * {at} assumed valid for {df3->axes}.
 */

off_t df3CalcFilepointer(df3Handle *df3, df3Coord const at) {
  off_t const b = (off_t)df3->bpv,
              dx = (off_t)df3->axes.x,
              dy = (off_t)df3->axes.y;
  return DF3_HEADERSIZE + (b * (off_t)at.z * dx * dy)
                        + (b * (off_t)at.y * dx)
                        + (b * (off_t)at.x);
}

/* ----------------------------------------------------------------- *
 *  this file is part of 'libdf3', part of the 'df3 tools' project.  *
 *  the content above is covered by the GPLv3+, see file COPYING.    *
 *  copyright (c) 2017 jr <creature.eternal@gmail.com>.              *
 *  all rights reserved.                                             *
 * ----------------------------------------------------------------- */
